"""Pack to handle web_apps setups.

This simply sets the server's default_path, if a web application is set as the default.
"""
from jinja2 import Environment

from ..utils import merge_settings


def process_settings(settings: dict) -> dict:
    """Process the env core settings.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :return: The updated settings
    :rtype: dict
    """
    if len(settings['web_apps']) > 0:
        settings = merge_settings(settings, {
            'packages': {
                'pip': [
                    'jupyter-server-proxy>=3.2.1<4.0.0'
                ]
            }
        })
    for web_app in settings['web_apps']:
        if 'default' in web_app:
            settings = merge_settings(settings, {
                'server': {
                    'default_path': web_app['path']
                }
            })
    return settings


def generate_files(context: str, env: Environment, settings: dict) -> None:
    """Generate the build files for the web_apps core.

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param settings: The validated settings
    :type settings: dict
    """
    pass



"""Pack to handle setting up the startup process in the container.

Generates the startup script and the sudoers file to ensure the user can chown their home-directory.
"""
import os.path

from jinja2 import Environment

from ..utils import merge_settings, render_template


def process_settings(settings: dict) -> dict:
    """Process the startup core settings.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :return: The updated settings
    :rtype: dict
    """
    settings = merge_settings(settings, {
        'content': [
            {
                'source': 'ou-builder-build/startup/start.sh',
                'target': '/usr/bin/start.sh',
                'overwrite': 'always'
            },
            {
                'source': 'ou-builder-build/startup/home-dir.sudoers',
                'target': '/etc/sudoers.d/99-home-dir',
                'overwrite': 'always'
            }
        ],
        'scripts': {
            'build': [
                {
                    'commands': [
                        'chmod a+x /usr/bin/start.sh'
                    ]
                }
            ]
        }
    })
    return settings


def generate_files(context: str, env: Environment, settings: dict) -> dict:
    """Generate the build files for the startup core.

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param settings: The validated settings
    :type settings: dict
    """
    # Generate the start script
    render_template(context,
                    env,
                    'core/startup/start.sh',
                    os.path.join('ou-builder-build', 'startup', 'start.sh'),
                    settings)
    # Generate the home-dir chown sudoers
    render_template(context,
                    env,
                    'core/startup/home-dir.sudoers',
                    os.path.join('ou-builder-build', 'startup', 'home-dir.sudoers'),
                    settings)



"""The main OU Container Builder commandline application.

Run ``ou-container-builder --help`` for help with the command-line parameters.
"""
import os
import shutil
import subprocess

from jinja2 import Environment, PackageLoader

from . import packs, core
from .validator import validate_settings
from .utils import merge_settings


def clean_build(context: str) -> None:
    """Clean the build files.

    :param context: The context directory containing the build files.
    :type context: str
    """
    if os.path.exists(os.path.join(context, 'Dockerfile')):
        os.unlink(os.path.join(context, 'Dockerfile'))
    if os.path.exists(os.path.join(context, 'ou-builder-build')):
        shutil.rmtree(os.path.join(context, 'ou-builder-build'))


def process_settings(settings: dict) -> dict:
    """Process the settings.

    :param settings: The settings provided by the user on the command-line
    :type settings: dict
    :return: The validated settings
    :rtype: dict
    """
    settings = validate_settings(settings)
    # Setup the base settings
    settings = merge_settings({}, settings)
    # Handle optional packs
    if 'packs' in settings:
        for pack in settings['packs']:
            if pack['name'] == 'code_server':
                settings = packs.code_server.process_settings(settings, pack['options'])
            elif pack['name'] == 'jupyterlab':
                settings = packs.jupyterlab.process_settings(settings, pack['options'])
            elif pack['name'] == 'mariadb':
                settings = packs.mariadb.process_settings(settings, pack['options'])
            elif pack['name'] == 'nbclassic':
                settings = packs.nbclassic.process_settings(settings, pack['options'])
            elif pack['name'] == 'tutorial_server':
                settings = packs.tutorial_server.process_settings(settings, pack['options'])
    # Handle core packs
    settings = core.env.process_settings(settings)
    if 'services' in settings:
        settings = core.services.process_settings(settings)
    if 'scripts' in settings:
        settings = core.scripts.process_settings(settings)
    if 'web_apps' in settings:
        settings = core.web_apps.process_settings(settings)
    settings = core.hacks.process_settings(settings)
    settings = core.jupyter_server.process_settings(settings)
    settings = core.startup.process_settings(settings)
    settings = core.content.process_settings(settings)
    settings = core.packages.process_settings(settings)
    return validate_settings(settings)


def generate_files(settings: dict, context: str, env: Environment) -> None:
    """Generate the files required for the Docker build.

    :param settings: The validated settings
    :type settings: dict
    :param context: The context directory
    :type context: str
    :param en: The jinja2 environment
    :type env: :class:`~jinja2.environment.Environment`
    """
    if 'packs' in settings:
        for pack in settings['packs']:
            if pack['name'] == 'code_server':
                packs.code_server.generate_files(context, env, settings, pack['options'])
            elif pack['name'] == 'jupyterlab':
                packs.jupyterlab.generate_files(context, env, settings, pack['options'])
            elif pack['name'] == 'mariadb':
                packs.mariadb.generate_files(context, env, settings, pack['options'])
            elif pack['name'] == 'nbclassic':
                packs.nbclassic.generate_files(context, env, settings, pack['options'])
            elif pack['name'] == 'tutorial_server':
                packs.tutorial_server.generate_files(context, env, settings, pack['options'])
    core.env.generate_files(context, env, settings)
    if 'services' in settings:
        core.services.generate_files(context, env, settings)
    if 'scripts' in settings:
        core.scripts.generate_files(context, env, settings)
    if 'web_apps' in settings:
        core.web_apps.generate_files(context, env, settings)
    core.hacks.generate_files(context, env, settings)
    core.jupyter_server.generate_files(context, env, settings)
    core.startup.generate_files(context, env, settings)
    core.content.generate_files(context, env, settings)
    core.packages.generate_files(context, env, settings)

    # Generate the Dockerfile
    with open(os.path.join(context, 'Dockerfile'), 'w') as out_f:
        tmpl = env.get_template('Dockerfile.jinja2')
        out_f.write(tmpl.render(**settings))


def docker_build(context: str, tag: list) -> None:
    """Run the docker build.

    :param context: The build context directory
    :type context: str
    :param tag: The tags to apply to the Docker image
    :type tag: list
    """
    cmd = ['docker', 'build', context]
    if tag:
        for t in tag:
            cmd.append('--tag')
            cmd.append(t)
    subprocess.run(cmd)


def run_build(settings: dict, context: str, build: bool, clean: bool, tag: list) -> None:
    """Run the build process.

    This processes the ``settings``, generates the required files, and then runs the Docker build process.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :param context: The directory within which to run the build
    :type context: str
    :param build: Whether to automatically invoke ``docker build``
    :type build: bool
    :param clean: Whether to automatically clean all generated files
    :type clean: bool
    :param tag: A list of tags to pass to docker
    :type tag: list[str]
    :return: A list with any errors that occured during processing
    :rtype: list
    """
    # Process and validate the settings
    settings = process_settings(settings)
    # Setup the build environment
    env = Environment(loader=PackageLoader('ou_container_builder', 'templates'), autoescape=False)
    clean_build(context)
    os.makedirs(os.path.join(context, 'ou-builder-build'))
    # Run the file generation
    generate_files(settings, context, env)

    if build:
        docker_build(context, tag)
        if clean:
            clean_build(context)



"""Pack to handle setting up jupyter_server."""
import json
import os.path

from jinja2 import Environment

from ..utils import merge_settings


def process_settings(settings: dict) -> dict:
    """Process the jupyter_server core settings.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :return: The updated settings
    :rtype: dict
    """
    settings = merge_settings(settings, {
        'packages': {
            'apt': [
                'sudo'
            ],
            'pip': [
                'jupyter_server<2.0.0',
                'jupyterhub<1.6.0'
            ]
        },
        'content': [
            {
                'source': 'ou-builder-build/ou_core_config.json',
                'target': '/usr/local/etc/jupyter/jupyter_server_config.json',
                'overwrite': 'always'
            }
        ],
        'jupyter_server_config': {
            'ServerApp': {
                'ip': '0.0.0.0',
                'port': 8888,
                'quit_button': False,
                'trust_xheaders': True,
                'iopub_data_rate_limit': 10000000
            },
            'NotebookApp': {
                'quite_button': False
            }
        }
    })
    return settings


def generate_files(context: str, env: Environment, settings: dict) -> dict:
    """Generate the build files the jupyter_server core.

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param settings: The validated settings
    :type settings: dict
    """
    settings = merge_settings(settings, {
        'jupyter_server_config': {
            'ServerApp': {
                'default_url': settings['server']['default_path']
            }
        }
    })
    if 'access_token' in settings['server'] and settings['server']['access_token']:
        settings = merge_settings(settings, {
            'jupyter_server_config': {
                'ServerApp': {
                    'token': settings['server']['access_token']
                }
            }
        })
    if 'wrapper_host' in settings['server'] and settings['server']['wrapper_host']:
        settings = merge_settings(settings, {
            'jupyter_server_config': {
                'ServerApp': {
                    'tornado_settings': {
                        'headers': {
                            'Content-Security-Policy': f"frame-ancestors 'self' {settings['server']['wrapper_host']}",
                            'Access-Control-Allow-Origin': f'{settings["server"]["wrapper_host"]}',
                        }
                    }
                }
            }
        })
    if 'web_apps' in settings:
        servers = {}
        for app in settings['web_apps']:
            app_config = {
                'command': app['cmdline'],
            }
            if 'port' in app:
                app_config['port'] = app['port']
            if 'timeout' in app:
                app_config['timeout'] = app['timeout']
            if 'absolute_url' in app:
                app_config['absolute_url'] = app['absolute_url']
            servers[app['path']] = app_config
        settings = merge_settings(settings, {
            'jupyter_server_config': {
                'ServerProxy': {
                    'servers': servers
                }
            }
        })
    with open(os.path.join(context, 'ou-builder-build', 'ou_core_config.json'), 'w') as out_f:
        json.dump(settings['jupyter_server_config'], out_f)



"""The main OU Container Builder commandline application.

Run ``ou-container-builder --help`` for help with the command-line parameters.
"""
import click
import os

from yaml import safe_load

from .builder import run_build


@click.command()
@click.option('-c', '--context',
              default='.',
              help='Context within which the container will be built',
              show_default=True)
@click.option('-b/-nb', '--build/--no-build',
              default=True,
              help='Automatically build the container',
              show_default=True)
@click.option('--clean/--no-clean',
              default=True,
              help='Automatically clean up after building the container',
              show_default=True)
@click.option('--tag',
              multiple=True,
              help='Automatically tag the generated image')
def main(context: str, build: bool, clean: bool, tag: list):
    """Build your OU Container."""
    with open(os.path.join(context, 'ContainerConfig.yaml')) as config_f:
        settings = safe_load(config_f)
    result = run_build(settings, context, build, clean, tag)
    if result:
        click.echo(click.style('There are errors in your configuration settings:', fg='red'), err=True)
        click.echo(err=True)
        for error in result:
            click.echo(error, err=True)


if __name__ == '__main__':
    main()



"""Utility functions."""
import os

from copy import deepcopy
from jinja2 import Environment


def merge_settings(base: dict, new: dict) -> dict:
    """Return a new dictionary created by merging the settings from ``new`` into ``base``.

    :param base: The base dictionary to merge into
    :type base: ``dict``
    :param new: The new dictionary to merge
    :type new: ``dict``
    :return: A new merged dictionary
    :rtype: ``dict``
    """
    result = {}

    for base_key, base_value in list(base.items()):
        if base_key not in new:
            result[base_key] = deepcopy(base_value)
        else:
            if isinstance(base_value, list):
                result[base_key] = list(base_value + new[base_key])
            elif isinstance(base_value, dict):
                result[base_key] = merge_settings(base_value, new[base_key])
            else:
                result[base_key] = new[base_key]
    for new_key, new_value in list(new.items()):
        if new_key not in base:
            result[new_key] = deepcopy(new_value)
    return result


def render_template(context: str, env: Environment, template_path: str, output_path: str, variables: dict) -> None:
    """Render a jinja2 template.

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param template_path: The path to the template to render
    :type template_path: str
    :param output_path: The path to output the result to
    :type output_path: str
    :param variables: The variables to pass to the template
    :type variables: dict
    """
    target_path = os.path.join(context, output_path)
    if not os.path.exists(os.path.dirname(target_path)):
        os.makedirs(os.path.dirname(target_path), exist_ok=True)
    with open(target_path, 'w') as out_f:
        tmpl = env.get_template(template_path)
        out_f.write(tmpl.render(**variables))



"""Pack to handle built-in hacks."""
from jinja2 import Environment


def process_settings(settings: dict) -> dict:
    """Process the hacks core settings.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :return: The updated settings
    :rtype: dict
    """
    if 'packages' in settings and 'apt' in settings['packages']:
        if 'openjdk-11-jdk' in settings['packages']['apt']:
            if 'hacks' in settings:
                if 'missing-man1' not in settings['hacks']:
                    settings['hacks'].append('missing-man1')
            else:
                settings['hacks'] = ['missing-man1']
    return settings


def generate_files(context: str, env: Environment, settings: dict) -> dict:
    """Generate the build files for the hacks core.

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param settings: The validated settings
    :type settings: dict
    """
    pass



"""Pack to install the Code Server interface."""
from jinja2 import Environment

from ..utils import merge_settings


def process_settings(settings: dict, pack_settings: dict) -> dict:
    """
    Process the user-provided settings.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :param pack_settings: The pack-specific settings parsed from the configuration file
    :type settings: dict
    :return: The updated settings
    :rtype: dict
    """
    settings = merge_settings(settings, {
        'sources': {
            'apt': [
                {
                    'name': 'nodesource',
                    'key_url': 'https://deb.nodesource.com/gpgkey/nodesource.gpg.key',
                    'deb': {
                        'url': 'https://deb.nodesource.com/node_18.x',
                        'distribution': 'bullseye',
                        'component': 'main'
                    }
                }
            ],
        },
        'packages': {
            'apt': [
                'nodejs',
                'yarn'
            ],
        },
        'scripts': {
            'build': [
                {
                    'commands': [
                        'cd /opt',
                        'wget https://github.com/coder/code-server/releases/download/v4.5.0/code-server-4.5.0-linux-amd64.tar.gz',  # noqa: E501
                        'tar -zxvf code-server-4.5.0-linux-amd64.tar.gz',
                        'ln -s /opt/code-server-4.5.0-linux-amd64/bin/code-server /usr/local/bin',
                    ]
                }
            ]
        },
        'web_apps': [
            {
                'path': '/code-server',
                'cmdline': [
                    'code-server',
                    '--auth',
                    'none',
                    '--disable-update-check',
                    '--bind-addr',
                    '0.0.0.0',
                    '--port',
                    '{port}'
                ],
                'timeout': 60,
                'default': True
            }
        ],
    })
    return settings


def generate_files(context: str, env: Environment, settings: dict, pack_settings: dict) -> None:
    """Generate the build files for the code-server pack.

    This ensures that the the code-server is installed

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param settings: The validated settings
    :type settings: dict
    :param pack_settings: The validated pack-specific settings
    :type settings: dict
    """
    pass



"""Pack to handle scripts."""
import os

from hashlib import sha256
from jinja2 import Environment

from ..utils import merge_settings


def process_settings(settings: dict) -> dict:
    """Process the env core settings.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :return: The updated settings
    :rtype: dict
    """
    if 'startup' in settings['scripts']:
        settings = merge_settings(settings, {
            'flags': {
                'ou_container_content': True
            }
        })
        new_settings = {}
        for script in settings['scripts']['startup']:
            if len(script['commands']) > 1:
                hash = sha256('\n'.join(script['commands']).encode())
                script_name = f'startup-script-{hash.hexdigest()}'
                new_settings = merge_settings(new_settings, {
                    'content': [
                        {
                            'source': os.path.join('ou-builder-build', script_name),
                            'target': f'/usr/bin/{script_name}',
                            'overwrite': 'always'
                        }
                    ],
                    'scripts': {
                        'build': [
                            {
                                'commands': [f'chmod a+x /usr/bin/{script_name}']
                            }
                        ]
                    }
                })
                script['commands'] = [
                    f'/usr/bin/{script_name}'
                ]
        settings = merge_settings(settings, new_settings)
    if 'shutdown' in settings['scripts']:
        settings = merge_settings(settings, {
            'flags': {
                'ou_container_content': True
            }
        })
        new_settings = {}
        for script in settings['scripts']['shutdown']:
            if len(script['commands']) > 1:
                hash = sha256('\n'.join(script['commands']).encode())
                script_name = f'shutdown-script-{hash.hexdigest()}'
                new_settings = merge_settings(new_settings, {
                    'content': [
                        {
                            'source': os.path.join('ou-builder-build', script_name),
                            'target': f'/usr/bin/{script_name}',
                            'overwrite': 'always'
                        }
                    ],
                    'scripts': {
                        'build': [
                            {
                                'commands': [f'chmod a+x /usr/bin/{script_name}']
                            }
                        ]
                    }
                })
                script['commands'] = [
                    f'/usr/bin/{script_name}'
                ]
        settings = merge_settings(settings, new_settings)

    return settings


def generate_files(context: str, env: Environment, settings: dict) -> dict:
    """Generate the build files for the scripts core.

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param settings: The validated settings
    :type settings: dict
    """
    if 'startup' in settings['scripts']:
        for script in settings['scripts']['startup']:
            if len(script['commands']) > 1:
                hash = sha256('\n'.join(script['commands']).encode())
                script_name = f'startup-script-{hash.hexdigest()}'
                with open(os.path.join(context, 'ou-builder-build', script_name), 'w') as out_f:
                    out_f.write('\n'.join(script['commands']))
    if 'shutdown' in settings['scripts']:
        for script in settings['scripts']['shutdown']:
            if len(script['commands']) > 1:
                hash = sha256('\n'.join(script['commands']).encode())
                script_name = f'shutdown-script-{hash.hexdigest()}'
                with open(os.path.join(context, 'ou-builder-build', script_name), 'w') as out_f:
                    out_f.write('\n'.join(script['commands']))



"""Pack to install the Jupyterlab interface."""
from jinja2 import Environment

from ..utils import merge_settings


def process_settings(settings: dict, pack_settings: dict) -> dict:
    """
    Process the user-provided settings.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :param pack_settings: The pack-specific settings parsed from the configuration file
    :type settings: dict
    :return: The updated settings
    :rtype: dict
    """
    settings = merge_settings(settings, {
        'packages': {
            'pip': ['jupyterlab>=3.4.3<4.0.0']
        },
        'server': {
            'default_path': '/lab'
        },
    })
    return settings


def generate_files(context: str, env: Environment, settings: dict, pack_settings: dict) -> None:
    """Generate the build files for the jupyterlab pack.

    This ensures that the the jupyterlab package is installed

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param settings: The validated settings
    :type settings: dict
    :param pack_settings: The validated pack-specific settings
    :type settings: dict
    """
    pass



"""Pack to install the tutorial_sever."""
import os

from jinja2 import Environment

from ..utils import merge_settings, render_template


def process_settings(settings: dict, pack_settings: dict) -> dict:
    """
    Process the user-provided settings.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :param pack_settings: The pack-specific settings parsed from the configuration file
    :type settings: dict
    :return: The updated settings
    :rtype: dict
    """
    additional_settings = {
        'packages': {
            'pip': [
                'tutorial-server>=1.0.2<2.0.0',
            ]
        },
        'content': [
            {
                'source': 'ou-builder-build/tutorial-server/production.ini',
                'target': '/etc/tutorial-server/production.ini',
                'overwrite': 'always'
            }
        ],
        'web_apps': [
            {
                'path': '/tutorial-server',
                'cmdline': [
                    'python',
                    '-m',
                    'tutorial_server',
                    '--config=/etc/tutorial-server/production.ini',
                    '--port={port}',
                    '--basepath={base_url}tutorial-server/'
                ],
                'absolute_url': True,
                'default': True
            }
        ]
    }
    if 'php-cgi' in pack_settings and pack_settings['php-cgi']:
        additional_settings['packages']['apt'] = ['php-cgi']
    settings = merge_settings(settings, additional_settings)
    return settings


def generate_files(context: str, env: Environment, settings: dict, pack_settings: dict) -> dict:
    """Generate the build files for the tutorial-server pack.

    Ensures that the Tutorial Server is installed and set up via the ``web_apps`` setting.

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param settings: The validated settings
    :type settings: dict
    :param pack_settings: The validated pack-specific settings
    :type settings: dict
    """
    render_template(context,
                    env,
                    'packs/tutorial-server/production.ini',
                    os.path.join('ou-builder-build', 'tutorial-server', 'production.ini'),
                    dict(**settings, **pack_settings))



"""Pack to handle setting up environment variables in the container."""
from jinja2 import Environment


def process_settings(settings: dict) -> dict:
    """Process the env core settings.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :return: The updated settings
    :rtype: dict
    """
    return settings


def generate_files(context: str, env: Environment, settings: dict) -> dict:
    """Generate the build files for the env core.

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param settings: The validated settings
    :type settings: dict
    """
    pass



"""Pack to install the MariaDB database."""
import os

from jinja2 import Environment

from ..utils import merge_settings, render_template


def process_settings(settings: dict, pack_settings: dict) -> dict:
    """Process the user-provided settings.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :param pack_settings: The pack-specific settings parsed from the configuration file
    :type settings: dict
    :return: The updated settings
    :rtype: dict
    """
    settings = merge_settings(settings, {
        'packages': {
            'apt': ['mariadb-server', 'sudo']
        },
        'scripts': {
            'build': [
                {
                    'commands': [
                        'mkdir -p /run/mysqld',
                        'sed -e "s#datadir.*=.*#datadir = $HOME/mariadb#" -e "s#user.*=.*#user = ou#" -i /etc/mysql/mariadb.conf.d/50-server.cnf',  # noqa: E501
                        'chown ou: /var/log/mysql/error.log /run/mysqld',
                        'chmod a+x /usr/bin/mariadb-setup.sh',
                        'printf "ou ALL=NOPASSWD: /usr/bin/mariadb-setup.sh\\n" > /etc/sudoers.d/99-mariadb'  # noqa: E501
                    ]
                },
            ],
            'startup': [
                {
                    'commands': [
                        'sudo /usr/bin/mariadb-setup.sh'
                    ]
                }
            ]
        },
        'services': [
            'mysql'
        ],
        'content': [
            {
                'source': '/var/lib/mysql',
                'target': 'mariadb',
                'overwrite': 'never'
            },
            {
                'source': 'ou-builder-build/mariadb/mariadb-setup.sh',
                'target': '/usr/bin/mariadb-setup.sh',
                'overwrite': 'always'
            }
        ]
    })


def generate_files(context: str, env: Environment, settings: dict, pack_settings: dict) -> None:
    """Generate the build files for the mariadb pack.

    This ensures that the the mariadb service is activated and that the configured database is set up in the user's
    home-directory.

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param settings: The validated settings
    :type settings: dict
    :param pack_settings: The validated pack-specific settings
    :type settings: dict
    """
    render_template(context,
                    env,
                    'packs/mariadb/setup.sh',
                    os.path.join('ou-builder-build', 'mariadb', 'mariadb-setup.sh'),
                    settings)



"""Pack to handle setting up content in the container.

This ensures that the OU Container Content application is integrated into the container and that all configured
files are distributed in the container as configured. This also sets the ``ou_container_content`` flag to ensure
that the OU Container Content application is run upon container startup.
"""
import os

from jinja2 import Environment

from ..utils import merge_settings, render_template


def process_settings(settings: dict) -> dict:
    """Process the content core settings.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :return: The updated settings
    :rtype: dict
    """
    if 'content' in settings and settings['content']:
        settings = merge_settings(settings, {
            'flags': {
                'ou_container_content': True
            }
        })
    if 'flags' in settings and settings['flags']:
        if 'ou_container_content' in settings['flags'] and settings['flags']['ou_container_content']:
            settings = merge_settings(settings, {
                'packages': {
                    'pip': [
                        'ou-container-content>=1.1.1'
                    ]
                },
                'scripts': {
                    'build': [
                        {
                            'commands': [
                                'ou-container-content prepare'
                            ]
                        }
                    ]
                }
            })
    return settings


def generate_files(context: str, env: Environment, settings: dict) -> None:
    """Generate the build files for the content core.

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param settings: The validated settings
    :type settings: dict
    """
    if 'flags' in settings and settings['flags']:
        if 'ou_container_content' in settings['flags'] and settings['flags']['ou_container_content']:
            render_template(context,
                            env,
                            'core/content/content_config.yaml',
                            os.path.join('ou-builder-build', 'content', 'content_config.yaml'),
                            settings)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ou_container_builder',
 'ou_container_builder.core',
 'ou_container_builder.packs']

package_data = \
{'': ['*'],
 'ou_container_builder': ['templates/*',
                          'templates/core/content/*',
                          'templates/core/services/*',
                          'templates/core/startup/*',
                          'templates/packs/mariadb/*',
                          'templates/packs/tutorial-server/*']}

install_requires = \
['cerberus>=1.3.3,<2.0.0',
 'click>=7.1.2,<8.0.0',
 'jinja2>=3.0.0,<4.0.0',
 'pyyaml>=5.4.1,<6.0.0']

entry_points = \
{'console_scripts': ['ou-container-builder = '
                     'ou_container_builder.__main__:main']}

setup_kwargs = {
    'name': 'ou-container-builder',
    'version': '2.0.0b15',
    'description': '',
    'long_description': None,
    'author': 'Mark Hall',
    'author_email': 'mark.hall@open.ac.uk',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
